@@ -1,4 +1,4 @@
-#  [ANDES] (C)2015-2020 Hantao Cui
+#  [ANDES] (C)2015-2021 Hantao Cui
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -266,6 +266,9 @@ class Block:
             if field.tex_name is None:
                 raise NameError(f'tex_name for <{field.name}> cannot be None')
 
+    def __repr__(self):
+        return f'{self.__class__.__name__}: {self.owner.__class__.__name__}.{self.name}'
+
 
 class PIController(Block):
     """
@@ -582,18 +585,22 @@ class Integrator(Block):
              └──────┘
 
     Exports a differential variable `y`.
-    The initial output is specified by `y0` and default to zero.
+
+    The initial output needs to be specified through `y0`.
     """
 
-    def __init__(self, u, T, K, y0, name=None, tex_name=None, info=None):
+    def __init__(self, u, T, K, y0, check_init=True,
+                 name=None, tex_name=None, info=None):
         super().__init__(name=name, tex_name=tex_name, info=info)
         self.u = dummify(u)
         self.K = dummify(K)
         self.T = dummify(T)
         self.y0 = dummify(y0)
+        self.check_init = check_init
         self.enforce_tex_name((self.K, self.T))
 
-        self.y = State(info='Integrator output', tex_name='y', t_const=self.T)
+        self.y = State(info='Integrator output', tex_name='y', t_const=self.T,
+                       check_init=check_init)
         self.vars = {'y': self.y}
 
     def define(self):
@@ -900,7 +907,10 @@ class LagAntiWindup(Block):
 class LagAWFreeze(LagAntiWindup):
     """
     Lag with anti-windup limiter and state freeze.
+
+    The output `y` is a state variable.
     """
+
     def __init__(self, u, T, K, lower, upper, freeze,
                  name=None, tex_name=None, info=None):
         LagAntiWindup.__init__(self, u, T, K, lower, upper,
@@ -1200,8 +1210,8 @@ class LeadLag(Block):
             x'^{(0)} & = u\\
             y^{(0)} & = Ku\\
 
-
         """
+
         self.x.v_str = f'{self.u.name}'
         self.y.v_str = f'{self.u.name}'
 
@@ -1542,6 +1552,7 @@ class LimiterGain(Block):
     """
 
     def __init__(self, u, K, lower, upper, no_lower=False, no_upper=False,
+                 sign_lower=1, sign_upper=1,
                  name=None, tex_name=None, info=None):
         Block.__init__(self, name=name, tex_name=tex_name, info=info)
         self.u = u
@@ -1557,6 +1568,7 @@ class LimiterGain(Block):
 
         self.lim = HardLimiter(u=self.u, lower=self.lower, upper=self.upper,
                                no_upper=no_upper, no_lower=no_lower,
+                               sign_lower=sign_lower, sign_upper=sign_upper,
                                tex_name='lim')
 
         self.y = Algeb(info='Gain output after limiter', tex_name='y', discrete=self.lim)
@@ -1571,11 +1583,11 @@ class LimiterGain(Block):
         self.y.v_str = f'{self.K.name} * {self.u.name} * {self.name}_lim_zi'
 
         if not self.no_upper:
-            self.y.e_str += f' + {self.K.name} * {self.name}_lim_zu*{self.upper.name}'
-            self.y.v_str += f' + {self.K.name} * {self.name}_lim_zu*{self.upper.name}'
+            self.y.e_str += f' + {self.K.name} * {self.name}_lim_zu*{self.upper.name} * {self.lim.sign_upper.name}'
+            self.y.v_str += f' + {self.K.name} * {self.name}_lim_zu*{self.upper.name} * {self.lim.sign_upper.name}'
         if not self.no_lower:
-            self.y.e_str += f' + {self.K.name} * {self.name}_lim_zl*{self.lower.name}'
-            self.y.v_str += f' + {self.K.name} * {self.name}_lim_zl*{self.lower.name}'
+            self.y.e_str += f' + {self.K.name} * {self.name}_lim_zl*{self.lower.name} * {self.lim.sign_lower.name}'
+            self.y.v_str += f' + {self.K.name} * {self.name}_lim_zl*{self.lower.name} * {self.lim.sign_lower.name}'
 
         self.y.e_str += f' - {self.name}_y'
 
@@ -1620,10 +1632,10 @@ class Piecewise(Block):
         i = 0
         for i in range(len(self.points)):
             args.append(f'({self.funs[i]}, {self.u.name} <= {self.points[i]})')
-        args.append(f'({self.funs[i + 1]}, True)')
+        args.append(f'({self.funs[i + 1]}, {self.u.name} > {self.points[-1]})')
 
         args_comma = ', '.join(args)
-        pw_fun = f'Piecewise({args_comma})'
+        pw_fun = f'Piecewise({args_comma}, evaluate=False)'
 
         self.y.v_str = pw_fun
         self.y.e_str = f'{pw_fun} - {self.name}_y'
@@ -1638,6 +1650,8 @@ class DeadBand1(Block):
     center
         Default value when within the deadband. If the input is an error signal,
         center should be set to zero.
+    gain
+        Gain multiplied to DeadBand discrete block's output.
 
     Notes
     -----
@@ -1645,12 +1659,12 @@ class DeadBand1(Block):
     Block diagram ::
 
               |   /
-        ______|__/___
+        ______|__/___   -> Gain -> DeadBand1_y
            /  |
           /   |
 
     """
-    def __init__(self, u, center, lower, upper, enable=True,
+    def __init__(self, u, center, lower, upper, gain=1.0, enable=True,
                  name=None, tex_name=None, info=None, namespace='local'):
         Block.__init__(self, name=name, tex_name=tex_name, info=info, namespace=namespace)
 
@@ -1658,6 +1672,7 @@ class DeadBand1(Block):
         self.center = dummify(center)
         self.lower = dummify(lower)
         self.upper = dummify(upper)
+        self.gain = dummify(gain)
         self.enable = enable
 
         self.db = DeadBand(u=u, center=center, lower=lower, upper=upper,
@@ -1675,8 +1690,8 @@ class DeadBand1(Block):
             0 = center + z_u * (u - upper) + z_l * (u - lower) - y
 
         """
-        self.y.v_str = f'{self.center.name} + ' \
+        self.y.v_str = f'{self.gain.name} * ({self.center.name} + ' \
                        f'{self.name}_db_zu * ({self.u.name} - {self.upper.name}) +' \
-                       f'{self.name}_db_zl * ({self.u.name} - {self.lower.name})'
+                       f'{self.name}_db_zl * ({self.u.name} - {self.lower.name}))'
 
         self.y.e_str = self.y.v_str + f' - {self.name}_y'
