@@ -1,7 +1,11 @@
 import pprint
-from collections import defaultdict, OrderedDict
-from typing import Iterable
 import logging
+
+from sympy import Expr
+
+from typing import Iterable
+from collections import defaultdict, OrderedDict
+
 from andes.shared import jac_full_names, jac_names, jac_types
 from andes.utils.tab import math_wrap, make_doc_table
 
@@ -207,7 +211,7 @@ class Config:
         """
         def warn_upper(s):
             if any(x.isupper() for x in s):
-                logger.warning(f"Config fields must be in lower case, found {s}")
+                logger.warning("Config fields must be in lower case, found %s", s)
 
         if dct is not None:
             for s in dct.keys():
@@ -236,7 +240,7 @@ class Config:
             kwargs.update(dct)
         for key, value in kwargs.items():
             if key not in self.__dict__:
-                logger.warning(f"Config field name {key} for {dest} is invalid.")
+                logger.warning("Config field name %s for %s is invalid.", key, dest)
                 continue
             self.__dict__[dest][key] = value
 
@@ -339,3 +343,19 @@ class Config:
     @property
     def tex_names(self):
         return self._tex
+
+
+class Indicator(Expr):
+    """
+    Indicator class for printing SymPy Relational.
+
+    Relational expressions in SymPy need to be wrapped by `Indicator`.
+
+    Examples
+    --------
+    To compare ``dae_t`` with ``0``, one need to use
+    ``Indicator(dae_t < 0)```.
+    """
+
+    def _numpycode(self, printer):
+        return printer._print(self.args[0])
