@@ -1,4 +1,4 @@
-#  [ANDES] (C)2015-2020 Hantao Cui
+#  [ANDES] (C)2015-2021 Hantao Cui
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -8,13 +8,15 @@
 #  File name: service.py
 #  Last modified: 8/16/20, 7:28 PM
 
+import logging
+import numpy as np
+
+from collections import OrderedDict
 from typing import Optional, Union, Callable, Type
+
 from andes.core.param import BaseParam
 from andes.utils.func import list_flatten
 from andes.core.common import dummify
-from andes.shared import np, ndarray
-import logging
-from collections import OrderedDict
 from andes.utils.tab import Tab
 
 logger = logging.getLogger(__name__)
@@ -24,7 +26,8 @@ class BaseService:
     """
     Base class for Service.
 
-    Service is a v-provider type for holding internal and temporary values. Subclasses need to implement ``v``
+    Service is a v-provider type for holding internal and temporary values.
+    Subclasses need to implement ``v``
     as a member attribute or using a property decorator.
 
     Parameters
@@ -42,7 +45,7 @@ class BaseService:
         self.name = name
         self.tex_name = tex_name if tex_name else name
         self.info = info
-        self.vtype = vtype if vtype is not None else np.float  # type for `v`
+        self.vtype = vtype if vtype is not None else float  # type for `v`
         self.owner = None
 
     def get_names(self):
@@ -56,6 +59,25 @@ class BaseService:
         """
         return [self.name]
 
+    def assign_memory(self, n):
+        """
+        Assign memory for ``self.v`` and set the array to zero.
+
+        Parameters
+        ----------
+        n : int
+            Number of elements of the value array.
+            Provided by caller (Model.list2array).
+        """
+        self.v = np.zeros(n, dtype=self.vtype)
+
+    @property
+    def class_name(self):
+        """
+        Return the class name
+        """
+        return self.__class__.__name__
+
     @property
     def n(self):
         """
@@ -73,13 +95,6 @@ class BaseService:
         else:
             return 1
 
-    @property
-    def class_name(self):
-        """
-        Return the class name
-        """
-        return self.__class__.__name__
-
     def __repr__(self):
         val_str = ''
 
@@ -120,11 +135,7 @@ class ConstService(BaseService):
         super().__init__(name=name, vtype=vtype, tex_name=tex_name, info=info)
         self.v_str = v_str
         self.v_numeric = v_numeric
-        self.v: Union[float, int, ndarray] = np.array([0.])
-
-    def assign_memory(self, n):
-        """Assign memory for ``self.v`` and set the array to zero."""
-        self.v = np.zeros(n, dtype=self.vtype)
+        self.v: Union[float, int, np.ndarray] = np.array([0.])
 
 
 class VarService(ConstService):
@@ -147,10 +158,11 @@ class VarService(ConstService):
 
     One can use `VarService` to implement this equation ::
 
-        self.VE = VarService(tex_name='V_E',
-                             info='VE',
-                             v_str='Abs(KPC*(vd + 1j*vq) + 1j*(KI + KPC*XL)*(Id + 1j*Iq))',
-                             )
+        self.VE = VarService(
+            tex_name='V_E',
+            info='VE',
+            v_str='Abs(KPC*(vd + 1j*vq) + 1j*(KI + KPC*XL)*(Id + 1j*Iq))',
+            )
 
     Warnings
     --------
@@ -192,6 +204,7 @@ class VarHold(VarService):
     """
     Service for holding the input when the hold state is on.
     """
+
     def __init__(self, u, hold, vtype=None, name=None, tex_name=None, info=None):
         VarService.__init__(self, v_numeric=self.check, vtype=vtype,
                             name=name, tex_name=tex_name, info=info,
@@ -474,10 +487,6 @@ class ExtService(BaseService):
         self.default = default
         self.v = np.array([0.])
 
-    def assign_memory(self, n):
-        """Assign memory for ``self.v`` and set the array to zero."""
-        self.v = np.zeros(n, dtype=self.vtype)
-
     def link_external(self, ext_model):
         """
         Method to be called by ``System`` for getting values from the external model or group.
@@ -581,27 +590,35 @@ class DeviceFinder(BaseService):
         self.link = link
 
     def find_or_add(self, system):
+        """
+        Find or add devices.
+
+        Points `self.u.v` to the found or newly added devices.
+
+        Find devices one by one.
+        Devices previously added in this function can be used later without duplication.
+        """
         mdl = system.models[self.model]
-        found_idx = mdl.find_idx((self.idx_name,), (self.link.v,),
-                                 allow_none=True, default=None)
+        added = False
+
+        for ii, link_to in enumerate(self.link.v):
+            idx = mdl.find_idx(self.idx_name, (link_to, ), allow_none=True, default=None)[0]
 
-        action = False
-        for ii, idx in enumerate(found_idx):
             if idx is None:
-                action = True
-                new_idx = system.add(self.model, {self.idx_name: self.link.v[ii]})
+                added = True
+                new_idx = system.add(self.model, {self.idx_name: link_to})
                 self.u.v[ii] = new_idx
 
                 logger.info(f"{self.owner.class_name} <{self.owner.idx.v[ii]}> "
                             f"added {self.model} <{new_idx}> "
-                            f"on {self.idx_name} <{self.link.v[ii]}>")
+                            f"linked to {self.idx_name} <{link_to}>")
             else:
-                action = True
                 self.u.v[ii] = idx
 
-        if action:
+        if added:
             mdl.list2array()
             mdl.refresh_inputs()
+            system.link_ext_param({mdl.name: mdl})
 
     @property
     def v(self):
@@ -610,7 +627,9 @@ class DeviceFinder(BaseService):
 
 class OperationService(BaseService):
     """
-    Base class for a type of Service which performs specific operations
+    Base class for a type of Service which performs specific operations.
+    OperationService may not use the `assign_memory` from `BaseService`,
+    because it can have a different size.
 
     This class cannot be used by itself.
 
@@ -900,6 +919,8 @@ class NumSelect(OperationService):
     """
     Class for selecting values for optional NumParam.
 
+    NumSelect works with internal and external parameters.
+
     Notes
     -----
     One use case is to allow an optional turbine rating. One can do ::
@@ -987,6 +1008,12 @@ class InitChecker(OperationService):
         self.enable = enable
         self.error_out = error_out
 
+        self.checks = [(self.lower, np.less_equal, "out of typical lower limit", "limit"),
+                       (self.upper, np.greater_equal, "out of typical upper limit", "limit"),
+                       (self.equal, lambda a, b: np.logical_not(np.isclose(a, b)), 'should be equal', "expected"),
+                       (self.not_equal, np.equal, 'should not be equal', "not expected")
+                       ]
+
     def check(self):
         """
         Check the bounds and equality conditions.
@@ -994,19 +1021,10 @@ class InitChecker(OperationService):
         if not self.enable:
             return
 
-        def _not_all_close(a, b):
-            return np.logical_not(np.isclose(a, b))
-
         if self._v is None:
             self._v = np.zeros_like(self.u.v)
 
-        checks = [(self.lower, np.less_equal, "violation of the lower limit", "limit"),
-                  (self.upper, np.greater_equal, "violation of the upper limit", "limit"),
-                  (self.equal, _not_all_close, 'should be equal', "expected"),
-                  (self.not_equal, np.equal, 'should not be equal', "not expected")
-                  ]
-
-        for check in checks:
+        for check in self.checks:
             limit = check[0]
             func = check[1]
             text = check[2]
@@ -1252,6 +1270,7 @@ class CurrentSign(ConstService):
         bus(+)        bus(-)
 
     """
+
     def __init__(self, bus, bus1, bus2,  name=None, tex_name=None, info=None):
         ConstService.__init__(self, v_numeric=self.check, name=name, tex_name=tex_name, info=info)
         self.bus = bus
@@ -1267,7 +1286,7 @@ class CurrentSign(ConstService):
             elif bus == bus2:
                 out[idx] = -1
             else:
-                raise ValueError(f"bus {bus} is terminal of the line connecting {bus1} and {bus2}. "
+                raise ValueError(f"bus {bus} is not terminal of the line connecting {bus1} and {bus2}. "
                                  f"Check the data of {self.bus.owner.class_name}.{self.bus.name}")
 
         return out
@@ -1363,3 +1382,143 @@ class RandomService(BaseService):
             Randomly generated service variables
         """
         return np.random.rand(self.n)
+
+
+class SwBlock(OperationService):
+    """
+    Service type for switched shunt blocks.
+    """
+
+    def __init__(self, *, init, ns, blocks, ext_sel=None,
+                 name=None, tex_name=None, info=None):
+        OperationService.__init__(self, name=name, tex_name=tex_name, info=info)
+        self.init = init
+        self.ns = ns
+        self.bs = blocks
+
+        self.sel = None
+        self.bcs = None
+        self.maxsel = None
+        self.ext_sel = ext_sel
+
+    @property
+    def v(self):
+        self.check_data()
+
+        # allocate memory
+        if self._v is not None:
+            return self._v
+
+        # initialize
+        n_dev = len(self.init.v)
+        self._v = np.array(self.init.v)           # effective value
+        self.sel = np.zeros(n_dev, dtype=int)     # the index of capacity in use
+        self.bcs = [0] * n_dev                    # cumulative sums of `bs`
+        self.maxsel = np.zeros(n_dev, dtype=int)  # max index into `bs`
+
+        for idx in range(n_dev):
+            item = self.ns.v[idx]
+            self.maxsel[idx] = sum(item)
+
+        # repeat each value in `bs` by `ns` times
+        for idx in range(n_dev):
+            # disabled - use default `b`
+            if self.maxsel[idx] == 0:
+                self.bcs[idx] = np.array([self.init.v[idx]])
+                continue
+
+            # calculate cumulative sum
+            b_rep = list([0])
+            for nn, bb in zip(self.ns.v[idx], self.bs.v[idx]):
+                b_rep += [bb] * nn
+            self.bcs[idx] = np.cumsum(b_rep)
+
+        if self.ext_sel is None:
+            # use internal selector - for shunt's b attribute
+            self.find_sel()
+        else:
+            # use external selector - for shunt's g attribute
+            self.sel = self.ext_sel.sel  # modify reference
+
+        self.set_v()
+        return self._v
+
+    def check_data(self):
+        """
+        Check data consistency.
+        """
+        model = self.ns.owner.class_name
+        for idx in range(len(self.ns.v)):
+            device = self.ns.owner.idx.v[idx]
+            bs_name = self.bs.name
+            ns_name = self.ns.name
+
+            if isinstance(self.ns.v[idx], (list, np.ndarray)):
+                if len(self.ns.v[idx]) == 0:
+                    continue
+                if len(self.ns.v[idx]) == 1 and self.ns.v[idx][0] == 0:
+                    continue
+
+            if isinstance(self.bs.v[idx], (int, float, str)):
+                raise ValueError("<%s>: idx=%s, `%s` parameter should be list literal, got %s" %
+                                 (model, device, bs_name, self.bs.v[idx]))
+
+            if len(self.ns.v[idx]) != len(self.bs.v[idx]):
+                raise ValueError("<%s>: idx=%s, `%s` and `%s` lengths do not match" %
+                                 (model, device, bs_name, ns_name))
+
+    def adjust(self, amount):
+        """
+        Adjust capacitor banks by an amount.
+        """
+        if self.ext_sel is None:
+            self.sel[:] += amount
+
+        self.set_v()
+
+    def set_v(self):
+        """
+        Set values to `_v` based on `sel`.
+        """
+
+        for idx in range(len(self._v)):
+            self._v[idx] = self.bcs[idx][self.sel[idx]]
+
+    def find_sel(self):
+        """
+        Determine the initial shunt selection level.
+        """
+        for idx in range(len(self._v)):
+            binit = self.init.v[idx]
+            if binit > self.bcs[idx][-1]:
+                # out of maximum b
+                logger.warning("<%s> idx=%s, initial %s=%g is greater than max=%g",
+                               self.init.owner.class_name,
+                               self.init.owner.idx.v[idx],
+                               self.init.name, binit,
+                               self.bcs[idx][-1])
+                self.sel[idx] = self.maxsel[idx]
+
+            elif binit == self.bcs[idx][-1]:
+                self.sel[idx] = self.maxsel[idx]
+
+            elif binit < 0:
+                logger.warning("<%s> idx=%s, initial %s=%g is less than zero",
+                               self.init.owner.class_name,
+                               self.init.owner.idx.v[idx],
+                               self.init.name, binit)
+                self.sel[idx] = 0
+
+            else:
+                for pos in range(self.maxsel[idx]):
+                    blo = self.bcs[idx][pos]
+                    bup = self.bcs[idx][pos + 1]
+                    if binit == blo:
+                        self.sel[idx] = pos
+                        break
+                    if binit == bup:
+                        self.sel[idx] = pos + 1
+                        break
+                    if blo < binit < bup:
+                        self.sel[idx] = pos + 1
+                        break
