@@ -33,35 +33,33 @@ class Streaming:
         """
         Connect to DiME 2 server.
 
-        If ``dime_address`` and ``dime_protocol`` are both specified
-        from command-line, streaming will be enabled.
+        If ``dime_address`` is specified from the command-line,
+        streaming will be automatically enabled.
         Otherwise, settings from the Config file will be used.
         """
         config = self.system.config
         options = self.system.options
 
         # enable only when both arguments are supplied
-        if options.get("dime_protocol") is not None and options.get("dime_address") is not None:
+        if options.get("dime_address") is not None:
 
             config.dime_enabled = True
-            config.dime_protocol = options.get("dime_protocol")
             config.dime_address = options.get("dime_address")
 
         if not config.dime_enabled:
             return False
 
         try:
-            self.dimec = DimeClient(config.dime_protocol, config.dime_address,
-                                    )
+            self.dimec = DimeClient(config.dime_address)
             self.dimec.join(config.dime_name)
-            logger.info(f"Dime connection to {config.dime_address} over {config.dime_protocol} was successful.")
+            logger.info('Dime connection to "%s" was successful.', config.dime_address)
             return True
         except NameError:
-            logger.warning('Dime not installed. Set System config `dime_enabled` to `0` to suppress warning.')
+            logger.error('Dime not installed. Set System config `dime_enabled` to `0` to suppress warning.')
             self.system.config.dime_enabled = False
 
         except FileNotFoundError:
-            logger.warning(f'Dime sever not found at "{config.dime_address}" over {config.dime_protocol}.')
+            logger.error('Dime sever not found at "%s".', config.dime_address)
             self.system.config.dime_enabled = False
 
         return False
@@ -97,8 +95,8 @@ class Streaming:
         }
         self.Idxvgs['Pmu'] = {
             # NOT YET SUPPORTED
-            # 'vm': 1 + array(self.system.PMU.vm),
-            # 'am': 1 + array(self.system.PMU.am),
+            'vm': 1 + self.system.PMU.vm.a,
+            'am': 1 + self.system.PMU.am.a,
         }
 
         # NOT YET SUPPORTED
@@ -285,8 +283,7 @@ class Streaming:
 
         self.ModuleInfo[name].update(ivar)
 
-        logger.debug('Module <{}> request index {}'.format(
-            name, var_idx))
+        logger.debug('Module <%s> requests index %s', name, var_idx)
 
     @staticmethod
     def transpose_matlab_row(a):
@@ -338,8 +335,8 @@ class Streaming:
                 param = {'tf': time, 'tc': tf, 'bus': idx}
                 self.system.Fault.insert(**param)
                 logger.debug(
-                    'Event <Fault> added for bus {} at t = {} and tf = {}'.
-                    format(idx, time, tf))
+                    'Event <Fault> added for bus %s at t = %.6g and tf = %g',
+                    idx, time, tf)
             elif name.lower() == 'line':
                 bus = self.system.Line.get_field(
                     'bus1', ['Line_' + str(int(idx - 1))])[0]
@@ -353,8 +350,8 @@ class Streaming:
                 }
                 self.system.Breaker.insert(**param)
                 logger.debug(
-                    'Event <Breaker> added for line {} at t = {} and tf = {}'.
-                    format(idx, time, tf))
+                    'Event <Breaker> added for line %s at t = %.6g and tf = %g',
+                    idx, time, tf)
 
             self.system.call.build_vec()
             self.system.call._compile_int()
@@ -418,7 +415,10 @@ class Streaming:
         if not self.has_pmu:
             return
 
-        idx = np.append(self.system.PMU.vm, self.system.PMU.am, self.system.BusFreq.w)
+        idx = np.concatenate((self.system.PMU.vm.a,
+                              self.system.PMU.am.a,
+                              self.system.dae.n + self.system.BusFreq.f.a,
+                              ))
 
         t = self.system.dae.t.tolist()
         k = 0  # field `k` is not no use
@@ -470,6 +470,7 @@ class Streaming:
             }
 
             self.dimec.send_r(mod, Varvgs=Varvgs)
+            logger.debug("Send Varvgs to module <%s>", mod)
 
     def finalize(self):
         """
