@@ -506,7 +506,7 @@ class RenGen(GroupBase):
     def __init__(self):
         super().__init__()
         self.common_params.extend(('bus', 'gen', 'Sn'))
-        self.common_vars.extend(('Ipcmd', 'Iqcmd', 'Pe', 'Qe'))
+        self.common_vars.extend(('Pe', 'Qe'))
 
 
 class RenExciter(GroupBase):
