@@ -2,7 +2,7 @@
 System class for power system data and methods
 """
 
-#  [ANDES] (C)2015-2020 Hantao Cui
+#  [ANDES] (C)2015-2021 Hantao Cui
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@ import sys
 import inspect
 import dill
 from collections import OrderedDict
-from typing import List, Dict, Tuple, Union, Optional
+from typing import Dict, Tuple, Union, Optional
 
 import andes.io
 from andes import __version__
@@ -30,25 +30,25 @@ from andes.variables import FileMan, DAE
 from andes.routines import all_routines
 from andes.utils.tab import Tab
 from andes.utils.misc import elapsed
-from andes.utils.paths import get_config_path, get_pkl_path, confirm_overwrite, get_dot_andes_path
+from andes.utils.paths import confirm_overwrite
+from andes.utils.paths import get_config_path, get_dot_andes_path
+from andes.utils.paths import get_pycode_path, get_pkl_path
 from andes.core import Config, Model, AntiWindup
 from andes.io.streaming import Streaming
 
-from andes.shared import np, spmatrix, jac_names, IP_ADD
+from andes.shared import np, jac_names, dilled_vars, IP_ADD
+from andes.shared import matrix, spmatrix, sparse
+
 logger = logging.getLogger(__name__)
-
 sys.path.append(get_dot_andes_path())
-
-try:
-    import pycode  # NOQA
-except ImportError:
-    pycode = None  # NOQA
+pycode = None
 
 
 class ExistingModels:
     """
     Storage class for existing models
     """
+
     def __init__(self):
         self.pflow = OrderedDict()
         self.tds = OrderedDict()   # if a model needs to be initialized before TDS, set `flags.tds = True`
@@ -86,9 +86,11 @@ class System:
     routines : OrderedDict
         routine name and instance pairs
     """
+
     def __init__(self,
                  case: Optional[str] = None,
                  name: Optional[str] = None,
+                 config: Optional[Dict] = None,
                  config_path: Optional[str] = None,
                  default_config: Optional[bool] = False,
                  options: Optional[Dict] = None,
@@ -117,7 +119,7 @@ class System:
             self._config_path = None
 
         self._config_object = self.load_config(self._config_path)
-        self.config = Config(self.__class__.__name__)
+        self.config = Config(self.__class__.__name__, dct=config)
         self.config.load(self._config_object)
 
         # custom configuration for system goes after this line
@@ -125,47 +127,53 @@ class System:
                                      ('mva', 100),
                                      ('store_z', 0),
                                      ('ipadd', 1),
+                                     ('seed', 'None'),
                                      ('diag_eps', 1e-8),
                                      ('warn_limits', 1),
                                      ('warn_abnormal', 1),
                                      ('dime_enabled', 0),
                                      ('dime_name', 'andes'),
-                                     ('dime_protocol', 'ipc'),
-                                     ('dime_address', '/tmp/dime2'),
+                                     ('dime_address', 'ipc:///tmp/dime2'),
                                      ('numba', 0),
                                      ('numba_parallel', 0),
-                                     ('save_pycode', 0),
                                      ('yapf_pycode', 0),
-                                     ('use_pycode', 0),
+                                     ('np_divide', 'warn'),
+                                     ('np_invalid', 'warn'),
+                                     ('pickle_path', get_pkl_path())
                                      )))
         self.config.add_extra("_help",
                               freq='base frequency [Hz]',
                               mva='system base MVA',
                               store_z='store limiter status in TDS output',
                               ipadd='use spmatrix.ipadd if available',
+                              seed='seed (or None) for random number generator',
                               diag_eps='small value for Jacobian diagonals',
                               warn_limits='warn variables initialized at limits',
                               warn_abnormal='warn initialization out of normal values',
                               numba='use numba for JIT compilation',
                               numba_parallel='enable parallel for numba.jit',
-                              save_pycode='save generated code to ~/.andes',
                               yapf_pycode='format generated code with yapf',
-                              use_pycode='use generated, saved Python code',
+                              np_divide='treatment for division by zero',
+                              np_invalid='treatment for invalid floating-point ops.',
+                              pickle_path='path models should be (un)dilled to/from',
                               )
         self.config.add_extra("_alt",
                               freq="float",
                               mva="float",
                               store_z=(0, 1),
                               ipadd=(0, 1),
+                              seed='int or None',
                               warn_limits=(0, 1),
                               warn_abnormal=(0, 1),
                               numba=(0, 1),
                               numba_parallel=(0, 1),
-                              save_pycode=(0, 1),
                               yapf_pycode=(0, 1),
-                              use_pycode=(0, 1),
+                              np_divide={'ignore', 'warn', 'raise', 'call', 'print', 'log'},
+                              np_invalid={'ignore', 'warn', 'raise', 'call', 'print', 'log'},
                               )
         self.config.check()
+        self._set_numpy()
+
         self.exist = ExistingModels()
 
         self.files = FileMan(case=case, **self.options)    # file path manager
@@ -181,9 +189,23 @@ class System:
         self._adders = dict(f=list(), g=list(), x=list(), y=list())
         self._setters = dict(f=list(), g=list(), x=list(), y=list())
         self.antiwindups = list()
+        self.no_check_init = list()  # states for which initialization check is omitted
 
         # internal flags
-        self.is_setup = False              # if system has been setup
+        self.is_setup = False        # if system has been setup
+
+    def _set_numpy(self):
+        """
+        Configure NumPy based on Config.
+        """
+        # set up numpy random seed
+        if isinstance(self.config.seed, int):
+            np.random.seed(self.config.seed)
+            logger.debug("Random seed set to <%d>.", self.config.seed)
+
+        np.seterr(divide=self.config.np_divide,
+                  invalid=self.config.np_invalid,
+                  )
 
     def reload(self, case, **kwargs):
         """
@@ -244,19 +266,22 @@ class System:
 
         # info
         if incremental is True:
-            text_mode = 'rapid incremental mode'
+            mode_text = 'rapid incremental mode'
         elif quick is True:
-            text_mode = 'quick mode'
+            mode_text = 'quick mode'
         else:
-            text_mode = 'full mode'
+            mode_text = 'full mode'
 
-        logger.info(f'Numerical code generation ({text_mode}) started...')
+        logger.info('Numerical code generation (%s) started...', mode_text)
 
         t0, _ = elapsed()
 
         # consistency check for group parameters and variables
         self._check_group_common()
 
+        # get `pycode` folder path without automatic creation
+        pycode_path = get_pycode_path(self.options.get("pycode_path"), mkdir=False)
+
         loaded_calls = self._load_pkl()
         if loaded_calls is None and incremental:
             incremental = False
@@ -276,29 +301,29 @@ class System:
 
             print(f"\r\x1b[K Generating code for {name} ({idx+1:>{width}}/{total:>{width}}).",
                   end='\r', flush=True)
-            model.prepare(quick=quick)
 
-        if self.config.save_pycode:
-            # write `__init__.py` that imports all to the `pycode` package
-            models_dir = os.path.join(get_dot_andes_path(), 'pycode')
-            os.makedirs(models_dir, exist_ok=True)
-            init_path = os.path.join(models_dir, '__init__.py')
+            model.prepare(quick=quick, pycode_path=pycode_path)
 
-            with open(init_path, 'w') as f:
-                f.write(f"__version__ = '{__version__}'\n\n")
-                for name in self.models.keys():
-                    f.write(f"from . import {name}  # NOQA\n")
-                f.write('\n')
+        # -- store pycode --
+        logger.info('Storing generated pycode to "%s"', pycode_path)
 
-            # RELOAD REQUIRED as the generated Jacobian arguments may be in a different order
-            if pycode is not None:
-                importlib.reload(pycode)
+        # write `__init__.py` that imports all to the `pycode` package
+        init_path = os.path.join(pycode_path, '__init__.py')
+        with open(init_path, 'w') as f:
+            f.write(f"__version__ = '{__version__}'\n\n")
+            for name in self.models.keys():
+                f.write(f"from . import {name}  # NOQA\n")
+            f.write('\n')
+
+        # RELOAD REQUIRED as the generated Jacobian arguments may be in a different order
+        if pycode is not None:
+            importlib.reload(pycode)
 
         self._store_calls()
         self.dill()
 
         _, s = elapsed(t0)
-        logger.info(f'Successfully generated numerical code in {s}.')
+        logger.info('Successfully generated numerical code in %s.', s)
 
     def _prepare_mp(self, quick=False):
         """
@@ -335,21 +360,24 @@ class System:
 
         This function is to be called after adding all device data.
         """
+        ret = True
+        t0, _ = elapsed()
+
         if self.is_setup:
             logger.warning('System has been setup. Calling setup twice is not allowed.')
-            return
+            ret = False
+            return ret
 
         self.collect_ref()
         self._list2array()     # `list2array` must come before `link_ext_param`
-        self.link_ext_param()
+        if not self.link_ext_param():
+            ret = False
+
         self.find_devices()    # find or add required devices
 
-        # == no device addition or removal after this point ==
-
-        self.calc_pu_coeff()
-
-        # store models with routine flags
-        self.store_existing()
+        # === no device addition or removal after this point ===
+        self.calc_pu_coeff()   # calculate parameters in system per units
+        self.store_existing()  # store models with routine flags
 
         # assign address at the end before adding devices and processing parameters
         self.set_address(self.exist.pflow)
@@ -357,7 +385,16 @@ class System:
         self.store_sparse_pattern(self.exist.pflow)
         self.store_adder_setter(self.exist.pflow)
 
-        self.is_setup = True
+        if ret is True:
+            self.is_setup = True  # set `is_setup` if no error occurred
+        else:
+            logger.error("System setup failed. Please resolve the reported issue(s).")
+            self.exit_code += 1
+
+        _, s = elapsed(t0)
+        logger.info('System internal structure set up in %s.', s)
+
+        return ret
 
     def store_existing(self):
         """
@@ -408,6 +445,9 @@ class System:
             param_dict.update(kwargs)
 
         idx = param_dict.pop('idx', None)
+        if idx is np.nan:
+            idx = None
+
         idx = group.get_next_idx(idx=idx, model_name=model)
         self.__dict__[model].add(idx=idx, **param_dict)
         group.add(idx=idx, model=self.__dict__[model])
@@ -429,15 +469,15 @@ class System:
         """
         Set addresses for differential and algebraic variables.
         """
-        # set internal variable addresses
         for mdl in models.values():
             if mdl.flags.address is True:
-                logger.debug(f'{mdl.class_name} internal address exists')
+                logger.debug('%s internal address exists', mdl.class_name)
                 continue
             if mdl.n == 0:
                 continue
 
-            logger.debug(f'Setting internal address for {mdl.class_name}')
+            # set internal variable addresses
+            logger.debug('Setting internal address for %s', mdl.class_name)
             n = mdl.n
             m0 = self.dae.m
             n0 = self.dae.n
@@ -458,7 +498,6 @@ class System:
 
             self.dae.m = m_end
             self.dae.n = n_end
-            mdl.flags.address = True
 
         # set external variable addresses
         for mdl in models.values():
@@ -470,25 +509,55 @@ class System:
                 except KeyError:
                     raise KeyError(f'<{ext_name}> is not a model or group name.')
 
-                instance.link_external(ext_model)
+                try:
+                    instance.link_external(ext_model)
+                except (IndexError, KeyError) as e:
+                    logger.error('Error: <%s> cannot retrieve <%s> from <%s> using <%s>:\n  %s',
+                                 mdl.class_name, instance.name, instance.model,
+                                 instance.indexer.name, repr(e))
+
+        # set external variable RHS addresses
+        for mdl in models.values():
+            if mdl.flags.address is True:
+                logger.debug('%s RHS address exists', mdl.class_name)
+                continue
+            if mdl.n == 0:
+                continue
+
+            for item in mdl.states_ext.values():
+                item.set_address(np.arange(self.dae.p, self.dae.p + item.n))
+                self.dae.p = self.dae.p + item.n
+            for item in mdl.algebs_ext.values():
+                item.set_address(np.arange(self.dae.q, self.dae.q + item.n))
+                self.dae.q = self.dae.q + item.n
+
+            mdl.flags.address = True
 
         # allocate memory for DAE arrays
         self.dae.resize_arrays()
 
         # set `v` and `e` in variables
-        self._set_var_arrays(models=models)
+        self.set_var_arrays(models=models)
 
         # pre-allocate for names
         if len(self.dae.y_name) == 0:
             self.dae.x_name = [''] * self.dae.n
             self.dae.y_name = [''] * self.dae.m
+            self.dae.h_name = [''] * self.dae.p
+            self.dae.i_name = [''] * self.dae.q
             self.dae.x_tex_name = [''] * self.dae.n
             self.dae.y_tex_name = [''] * self.dae.m
+            self.dae.h_tex_name = [''] * self.dae.p
+            self.dae.i_tex_name = [''] * self.dae.q
         else:
             self.dae.x_name.extend([''] * (self.dae.n - len(self.dae.x_name)))
             self.dae.y_name.extend([''] * (self.dae.m - len(self.dae.y_name)))
+            self.dae.h_name.extend([''] * (self.dae.p - len(self.dae.h_name)))
+            self.dae.i_name.extend([''] * (self.dae.q - len(self.dae.i_name)))
             self.dae.x_tex_name.extend([''] * (self.dae.n - len(self.dae.x_tex_name)))
             self.dae.y_tex_name.extend([''] * (self.dae.m - len(self.dae.y_tex_name)))
+            self.dae.h_tex_name.extend([''] * (self.dae.p - len(self.dae.h_tex_name)))
+            self.dae.i_tex_name.extend([''] * (self.dae.q - len(self.dae.i_tex_name)))
 
     def set_dae_names(self, models):
         """
@@ -528,7 +597,7 @@ class System:
                             self.dae.z_tex_name.append(rf'${item.tex_name}$ {append_model_name(mdl_name, id)}')
                             self.dae.o += 1
 
-    def _set_var_arrays(self, models):
+    def set_var_arrays(self, models, inplace=True, alloc=True):
         """
         Set arrays (`v` and `e`) in internal variables.
 
@@ -536,14 +605,34 @@ class System:
         ----------
         models : OrderedDict, list, Model, optional
             Models to execute.
-
+        inplace : bool
+            True to retrieve arrays that share memory with dae
+        alloc : bool
+            True to allocate for arrays internally
         """
+
         for mdl in models.values():
             if mdl.n == 0:
                 continue
 
             for var in mdl.cache.vars_int.values():
-                var.set_arrays(self.dae)
+                var.set_arrays(self.dae, inplace=inplace, alloc=alloc)
+
+            for var in mdl.cache.vars_ext.values():
+                var.set_arrays(self.dae, inplace=inplace, alloc=alloc)
+
+    def _init_numba(self, models: OrderedDict):
+        """
+        Helper function to compile all functions with Numba before init.
+        """
+        if self.config.numba:
+            use_parallel = True if (self.config.numba_parallel == 1) else False
+            use_cache = True if (pycode is not None) else False
+
+            logger.info("Numba compilation initiated, parallel=%s, cache=%s.",
+                        use_parallel, use_cache)
+            for mdl in models.values():
+                mdl.numba_jitify(parallel=use_parallel, cache=use_cache)
 
     def init(self, models: OrderedDict, routine: str):
         """
@@ -555,13 +644,10 @@ class System:
         - Call the model `init()` method, which initializes internal variables.
         - Copy variables to DAE and then back to the model.
         """
-        if self.config.numba:
-            use_parallel = True if (self.config.numba_parallel == 1) else False
-            use_cache = True if (pycode is not None) else False
-
-            logger.info(f"Numba compilation initiated, parallel={use_parallel}, cache={use_cache}.")
-            for mdl in models.values():
-                mdl.numba_jitify(parallel=use_parallel, cache=use_cache)
+        try:
+            self._init_numba(models)
+        except ImportError:
+            logger.warning("Numba not found. JIT compilation is skipped.")
 
         for mdl in models.values():
             # link externals first
@@ -570,9 +656,14 @@ class System:
                 try:
                     ext_model = self.__dict__[ext_name]
                 except KeyError:
-                    raise KeyError(f'<{ext_name}> is not a model or group name.')
+                    raise KeyError('<%s> is not a model or group name.' % ext_name)
 
-                instance.link_external(ext_model)
+                try:
+                    instance.link_external(ext_model)
+                except (IndexError, KeyError) as e:
+                    logger.error('Error: <%s> cannot retrieve <%s> from <%s> using <%s>:\n  %s',
+                                 mdl.class_name, instance.name, instance.model,
+                                 instance.indexer.name, repr(e))
 
             # initialize variables second
             mdl.init(routine=routine)
@@ -622,6 +713,19 @@ class System:
 
         return
 
+    def store_no_check_init(self, models):
+        """
+        Store differential variables with ``check_init == False``.
+        """
+        self.no_check_init = list()
+        for mdl in models.values():
+            if mdl.n == 0:
+                continue
+
+            for var in mdl.states.values():
+                if var.check_init is False:
+                    self.no_check_init.extend(var.a)
+
     def link_ext_param(self, model=None):
         """
         Retrieve values for ``ExtParam`` for the given models.
@@ -631,6 +735,7 @@ class System:
         else:
             models = self._get_models(model)
 
+        ret = True
         for model in models.values():
             # get external parameters with `link_external` and then calculate the pu coeff
             for instance in model.params_ext.values():
@@ -639,8 +744,12 @@ class System:
 
                 try:
                     instance.link_external(ext_model)
-                except IndexError:
-                    raise IndexError(f'Model {model.class_name}.{instance.name} link parameter error')
+                except (IndexError, KeyError) as e:
+                    logger.error('Error: <%s> cannot retrieve <%s> from <%s> using <%s>:\n  %s',
+                                 model.class_name, instance.name, instance.model,
+                                 instance.indexer.name, repr(e))
+                    ret = False
+        return ret
 
     def calc_pu_coeff(self):
         """
@@ -652,9 +761,6 @@ class System:
         Sb = self.config.mva
 
         for mdl in self.models.values():
-            # before this step, `link_ext_param` has been called in `setup`.
-            self.link_ext_param({mdl.class_name: mdl})
-
             # default Sn to Sb if not provided. Some controllers might not have Sn or Vn.
             if 'Sn' in mdl.__dict__:
                 Sn = mdl.Sn.v
@@ -704,13 +810,14 @@ class System:
                 for p in mdl.find_param(prop).values():
                     p.set_pu_coeff(coeff)
 
-    def l_update_var(self, models: OrderedDict):
+    def l_update_var(self, models: OrderedDict, niter=None, err=None):
         """
         Update variable-based limiter discrete states by calling ``l_update_var`` of models.
 
         This function is must be called before any equation evaluation.
         """
-        self.call_models('l_update_var', models, self.dae.t)
+        self.call_models('l_update_var', models,
+                         dae_t=self.dae.t, niter=niter, err=err)
 
     def l_update_eq(self, models:  OrderedDict):
         """
@@ -747,6 +854,9 @@ class System:
         """
         self._e_to_dae(('f', 'g'))
 
+        # reset mismatches for islanded buses
+        self.g_islands()
+
         # update variable values set by anti-windup limiters
         for item in self.antiwindups:
             if len(item.x_set) > 0:
@@ -781,6 +891,16 @@ class System:
             logger.error("g_update failed. Have you run `andes prepare -i` after updating?")
             raise e
 
+    def g_islands(self):
+        """
+        Reset algebraic mismatches for islanded buses.
+        """
+        if self.Bus.n_islanded_buses == 0:
+            return
+
+        self.dae.g[self.Bus.islanded_a] = 0.0
+        self.dae.g[self.Bus.islanded_v] = 0.0
+
     def j_update(self, models: OrderedDict, info=None):
         """
         Call the Jacobian update method for models in sequence.
@@ -813,11 +933,33 @@ class System:
                                      f'j_size={j_size}')
                         raise e
 
+        self.j_islands()
+
         if info:
             logger.debug("Jacobian updated at t=%.6f due to %s.", self.dae.t, info)
         else:
             logger.debug("Jacobian updated at t=%.6f.", self.dae.t)
 
+    def j_islands(self):
+        """
+        Set gy diagonals to eps for `a` and `v` variables of islanded buses.
+        """
+        if self.Bus.n_islanded_buses == 0:
+            return
+
+        aidx = self.Bus.islanded_a
+        vidx = self.Bus.islanded_v
+
+        if self.config.ipadd and IP_ADD:
+            self.dae.gy.ipset(self.config.diag_eps, aidx, aidx)
+            self.dae.gy.ipset(self.config.diag_eps, vidx, vidx)
+        else:
+            avals = [-self.dae.gy[int(idx), int(idx)] + self.config.diag_eps for idx in aidx]
+            vvals = [-self.dae.gy[int(idx), int(idx)] + self.config.diag_eps for idx in vidx]
+
+            self.dae.gy += spmatrix(avals, aidx, aidx, self.dae.gy.size, 'd')
+            self.dae.gy += spmatrix(vvals, vidx, vidx, self.dae.gy.size, 'd')
+
     def store_sparse_pattern(self, models: OrderedDict):
         """
         Collect and store the sparsity pattern of Jacobian matrices.
@@ -836,6 +978,7 @@ class System:
         for jname in jac_names:
             ii, jj, vv = list(), list(), list()
 
+            # for `gy`, reserve memory for the main diagonal
             if jname == 'gy':
                 ii.extend(np.arange(self.dae.m))
                 jj.extend(np.arange(self.dae.m))
@@ -882,11 +1025,186 @@ class System:
             if var.n > 0:
                 var.v[:] = self.dae.x[var.a]
 
+    def connectivity(self, info=True):
+        """
+        Perform connectivity check for system.
+
+        Parameters
+        ----------
+        info : bool
+            True to log connectivity summary.
+        """
+        logger.debug("Entering connectivity check.")
+
+        self.Bus.n_islanded_buses = 0
+        self.Bus.islanded_buses = list()
+        self.Bus.island_sets = list()
+        self.Bus.nosw_island = list()
+        self.Bus.msw_island = list()
+        self.Bus.islands = list()
+
+        n = self.Bus.n
+
+        # collect from-bus and to-bus indices
+        fr, to, u = list(), list(), list()
+
+        # collect from Line
+        fr.extend(self.Line.a1.a.tolist())
+        to.extend(self.Line.a2.a.tolist())
+        u.extend(self.Line.u.v.tolist())
+        os = [0] * len(u)
+
+        # find islanded buses
+        diag = list(matrix(spmatrix(u, to, os, (n, 1), 'd') +
+                           spmatrix(u, fr, os, (n, 1), 'd')))
+
+        nib = self.Bus.n_islanded_buses = diag.count(0)
+        for idx in range(n):
+            if diag[idx] == 0:
+                self.Bus.islanded_buses.append(idx)
+
+        # store `a` and `v` indices for zeroing out residuals
+        self.Bus.islanded_a = np.array(self.Bus.islanded_buses)
+        self.Bus.islanded_v = self.Bus.n + self.Bus.islanded_a
+
+        # find islanded areas - Goderya's algorithm
+        temp = spmatrix(list(u) * 4,
+                        fr + to + fr + to,
+                        to + fr + fr + to,
+                        (n, n),
+                        'd')
+
+        cons = temp[0, :]
+        nelm = len(cons.J)
+        conn = spmatrix([], [], [], (1, n), 'd')
+        enum = idx = islands = 0
+
+        while True:
+            while True:
+                cons = cons * temp
+                cons = sparse(cons)  # remove zero values
+                new_nelm = len(cons.J)
+                if new_nelm == nelm:
+                    break
+                nelm = new_nelm
+
+            # started with an islanded bus
+            if len(conn.J) == 0:
+                enum += 1
+            # all buses are interconnected
+            elif len(cons.J) == n:
+                break
+
+            self.Bus.island_sets.append(list(cons.J))
+            conn += cons
+            islands += 1
+            nconn = len(conn.J)
+            if nconn >= (n - nib):
+                self.Bus.island_sets = [i for i in self.Bus.island_sets if len(i) > 0]
+                break
+
+            for element in conn.J[idx:]:
+                if not diag[idx]:
+                    enum += 1  # skip islanded buses
+                if element <= enum:
+                    idx += 1
+                    enum += 1
+                else:
+                    break
+
+            cons = temp[enum, :]
+
+        # --- check if all areas have a slack generator ---
+        if len(self.Bus.island_sets) > 0:
+            for idx, island in enumerate(self.Bus.island_sets):
+                nosw = 1
+                slack_bus_uid = self.Bus.idx2uid(self.Slack.bus.v)
+                slack_u = self.Slack.u.v
+                for u, item in zip(slack_u, slack_bus_uid):
+                    if (u == 1) and (item in island):
+                        nosw -= 1
+                if nosw == 1:
+                    self.Bus.nosw_island.append(idx)
+                elif nosw < 0:
+                    self.Bus.msw_island.append(idx)
+
+        # --- Post processing ---
+        # extend islanded buses, each in a list
+        if len(self.Bus.islanded_buses) > 0:
+            self.Bus.islands.extend([[item] for item in self.Bus.islanded_buses])
+
+        if len(self.Bus.island_sets) == 0:
+            self.Bus.islands.append(list(range(n)))
+        else:
+            self.Bus.islands.extend(self.Bus.island_sets)
+
+        if info is True:
+            self.summary()
+
+    def to_ipysheet(self, model: str, vin: bool = False):
+        """
+        Return an ipysheet object for editing in Jupyter Notebook.
+        """
+
+        from ipysheet import from_dataframe
+
+        return from_dataframe(self.models[model].as_df(vin=vin))
+
+    def from_ipysheet(self, model: str, sheet, vin: bool = False):
+        """
+        Set an ipysheet object back to model.
+        """
+
+        from ipysheet import to_dataframe
+
+        df = to_dataframe(sheet)
+        self.models[model].update_from_df(df, vin=vin)
+
+    def summary(self):
+        """
+        Print out system summary.
+        """
+
+        island_sets = self.Bus.island_sets
+        nosw_island = self.Bus.nosw_island
+        msw_island = self.Bus.msw_island
+        n_islanded_buses = self.Bus.n_islanded_buses
+
+        logger.info("-> System connectivity check results:")
+        if n_islanded_buses == 0:
+            logger.info("  No islanded bus detected.")
+        else:
+            logger.info("  %d islanded bus detected.", n_islanded_buses)
+            logger.debug("  Islanded Bus indices (0-based): %s", self.Bus.islanded_buses)
+
+        if len(island_sets) == 0:
+            logger.info("  No island detected.")
+        else:
+            logger.info("  A total of %d island(s) detected.", len(island_sets))
+            logger.debug("  Bus indices in islanded areas (0-based): %s", island_sets)
+
+        if len(nosw_island) > 0:
+            logger.warning('  Slack generator is not defined/enabled for %d island(s).',
+                           len(nosw_island))
+            logger.debug("  Bus indices in no-Slack areas (0-based): %s",
+                         [island_sets[item] for item in nosw_island])
+
+        if len(msw_island) > 0:
+            logger.warning('  Multiple slack generators are defined/enabled for %d island(s).',
+                           len(msw_island))
+            logger.debug("  Bus indices in multiple-Slack areas (0-based): %s",
+                         [island_sets[item] for item in msw_island])
+
+        if len(self.Bus.nosw_island) == 0 and len(self.Bus.msw_island) == 0:
+            logger.info('  Each island has a slack bus correctly defined and enabled.')
+
     def _v_to_dae(self, v_code, model):
         """
-        Helper function for collecting variable values into dae structures `x` and `y`.
+        Helper function for collecting variable values into ``dae``
+        structures `x` and `y`.
 
-        This function must be called with x and y both being zeros.
+        This function must be called with ``dae.x`` and ``dae.y``
+        both being zeros.
         Otherwise, adders will be summed again, causing an error.
 
         Parameters
@@ -928,9 +1246,10 @@ class System:
             for var in self._setters[name]:
                 np.put(self.dae.__dict__[name], var.a, var.e)
 
-    def get_z(self, models: Optional[Union[str, List, OrderedDict]] = None):
+    def get_z(self, models: OrderedDict):
         """
         Get all discrete status flags in a numpy array.
+        Values are written to ``dae.z`` in place.
 
         Returns
         -------
@@ -939,13 +1258,24 @@ class System:
         if self.config.store_z != 1:
             return None
 
-        z_dict = list()
+        if len(self.dae.z) != self.dae.o:
+            self.dae.z = np.zeros(self.dae.o, dtype=float)
+
+        ii = 0
         for mdl in models.values():
             if mdl.n == 0 or len(mdl._input_z) == 0:
                 continue
-            z_dict.append(np.concatenate(list((mdl._input_z.values()))))
+            for zz in mdl._input_z.values():
+                self.dae.z[ii:ii + mdl.n] = zz
+                ii += mdl.n
 
-        return np.concatenate(z_dict)
+        return self.dae.z
+
+    def get_ext_fg(self, model: OrderedDict):
+        """
+        Get the right-hand side of the external equations.
+        """
+        pass
 
     def find_models(self, flag: Optional[Union[str, Tuple]], skip_zero: bool = True):
         """
@@ -997,33 +1327,17 @@ class System:
         This function sets `dill.settings['recurse'] = True` to serialize the function calls recursively.
 
         """
-        logger.debug("Dumping calls to calls.pkl with dill")
-        dill.settings['recurse'] = True
+        np_ver = np.__version__.split('.')
+        np_ver = tuple([int(i) for i in np_ver])
+        if np_ver < (1, 20):
+            logger.debug("Dumping calls to calls.pkl with dill")
+            dill.settings['recurse'] = True
 
-        pkl_path = get_pkl_path()
-        with open(pkl_path, 'wb') as f:
-            dill.dump(self.calls, f)
-
-    @staticmethod
-    def _load_pkl():
-        """
-        Helper function to open and load dill-pickled functions.
-        """
-        dill.settings['recurse'] = True
-        pkl_path = get_pkl_path()
-
-        if os.path.isfile(pkl_path):
-            with open(pkl_path, 'rb') as f:
-
-                try:
-                    loaded_calls = dill.load(f)
-                    return loaded_calls
-                except IOError:
-                    pass
-                except AttributeError:
-                    pass
-
-        return None
+            with open(self.config.pickle_path, 'wb') as f:
+                dill.dump(self.calls, f)
+        else:
+            logger.warning("Dumping calls to calls.pkl is not supported with NumPy 1.2+")
+            logger.warning("ANDES is still fully functional with generated Python code.")
 
     def undill(self):
         """
@@ -1033,38 +1347,118 @@ class System:
         acceleration.
         """
 
+        # try to replace equations and jacobian calls with saved code
+        calls_loaded = False
+
+        try:
+            self._call_from_pycode()
+            calls_loaded = True
+        except ImportError:
+            logger.debug("Pycode not found. Trying to load from `calls.pkl`.")
+
+        if calls_loaded is False:
+            self._call_from_pkl()
+
+    def _load_pkl(self):
+        """
+        Helper function to open and load dill-pickled functions.
+        """
+        dill.settings['recurse'] = True
+
+        if os.path.isfile(self.config.pickle_path):
+            with open(self.config.pickle_path, 'rb') as f:
+
+                try:
+                    loaded_calls = dill.load(f)
+                    return loaded_calls
+                except (IOError, EOFError, AttributeError):
+                    pass
+
+        return None
+
+    def _call_from_pkl(self):
+        """
+        Helper function for loading ModelCall from pickle file.
+        """
+
         loaded_calls = self._load_pkl()
 
         if loaded_calls is not None:
             ver = loaded_calls.get('__version__')
             if ver == __version__:
                 self.calls = loaded_calls
-                logger.debug(f'Undilled calls from "{get_pkl_path()}" is current.')
+                logger.info('Undilled calls from "%s" is up-to-date.', self.config.pickle_path)
             else:
-                logger.info(f'Undilled calls are for version {ver}, regenerating...')
+                logger.info('Undilled calls are for version %s, regenerating...', ver)
                 self.prepare(quick=True, incremental=True)
 
         else:
             logger.info('Generating numerical calls at the first launch.')
-            self.prepare()
+            self.prepare(quick=True)
 
         for name, model_call in self.calls.items():
             if name in self.__dict__:
                 self.__dict__[name].calls = model_call
 
-        # try to replace equations and jacobian calls with saved code
-        if pycode is not None and self.config.use_pycode:
-            for model in self.models.values():
-                pycode_model = pycode.__dict__[model.class_name]
+        logger.debug("Using undilled lambda functions.")
 
-                model.calls.f = pycode_model.__dict__.get("f_update")
-                model.calls.g = pycode_model.__dict__.get("g_update")
+    def _call_from_pycode(self):
+        """
+        Helper function to load generated pycode
+        """
+        pycode = None
+        try:
+            import pycode  # NOQA
+        except ImportError:
+            globals()['pycode'] = pycode
 
-                for jname in model.calls.j:
-                    model.calls.j[jname] = pycode_model.__dict__.get(f'{jname}_update')
-            logger.info("Using generated Python code for equations and Jacobians.")
-        else:
-            logger.debug("Using undilled lambda functions.")
+        try:
+            from andes import pycode  # NOQA
+        except ImportError:
+            globals()['pycode'] = pycode
+
+        if pycode is None:
+            raise ImportError("Generated pycode not found.")
+
+        for name in self.models:
+            if name not in pycode.__dict__:
+                raise ImportError("Generated pycode is incomplete.")
+
+        for model in self.models.values():
+            pycode_model = pycode.__dict__[model.class_name]
+
+            # reload stored variables
+            for item in dilled_vars:
+                model.calls.__dict__[item] = pycode_model.__dict__[item]
+
+            # equations
+            model.calls.f = pycode_model.__dict__.get("f_update")
+            model.calls.g = pycode_model.__dict__.get("g_update")
+
+            # services
+            for instance in model.services.values():
+                if instance.v_str is not None:
+                    sv_name = f'{instance.name}_svc'
+                    model.calls.s[instance.name] = pycode_model.__dict__[sv_name]
+
+            # load initialization; assignment
+            for instance in model.cache.all_vars.values():
+                if instance.v_str is not None:
+                    ia_name = f'{instance.name}_ia'
+                    model.calls.ia[instance.name] = pycode_model.__dict__[ia_name]
+
+            # load initialization: iterative
+            for item in model.calls.init_seq:
+                if isinstance(item, list):
+                    name_concat = '_'.join(item)
+                    model.calls.ii[name_concat] = pycode_model.__dict__[name_concat + '_ii']
+                    model.calls.ij[name_concat] = pycode_model.__dict__[name_concat + '_ij']
+
+            # load Jacobian functions
+            for jname in model.calls.j_names:
+                model.calls.j[jname] = pycode_model.__dict__.get(f'{jname}_update')
+
+        logger.info("Using generated Python code.")
 
     def _get_models(self, models):
         """
@@ -1278,7 +1672,7 @@ class System:
         array-like
             self.switch_times
         """
-        out = np.array([], dtype=np.float)
+        out = np.array([], dtype=float)
 
         if self.options.get('flat') is True:
             return out
@@ -1314,7 +1708,7 @@ class System:
         self.n_switches = len(self.switch_times)
         return self.switch_times
 
-    def switch_action(self, models):
+    def switch_action(self, models: OrderedDict):
         """
         Invoke the actions associated with switch times.
 
@@ -1418,7 +1812,7 @@ class System:
 
         conf = configparser.ConfigParser()
         conf.read(conf_path)
-        logger.info(f'Loaded config from file "{conf_path}"')
+        logger.info('Loaded config from file "%s"', conf_path)
         return conf
 
     def save_config(self, file_path=None, overwrite=False):
@@ -1451,7 +1845,7 @@ class System:
         with open(file_path, 'w') as f:
             conf.write(f)
 
-        logger.info(f'Config written to "{file_path}"')
+        logger.info('Config written to "%s"', file_path)
         return file_path
 
     def supported_models(self, export='plain'):
